==> enterprisemodules-ora_config-3.14.3.tar.gz: enterprisemodules-ora_config-3.14.3/tasks/insert_data_bu.rb <==
#!/opt/puppetlabs/puppet/bin/ruby
#
# See the file "LICENSE" for the full license governing this code.
#
require 'json'
require 'puppet'

begin
  Puppet.initialize_settings
rescue Puppet::DevError
  # When run in Rspec, this is called multiple times
end

class InsertDataBackup

  def self.run
    instance = self.new
    instance.run
  end

  def run()
    @starttime = Time.now.iso8601
    begin
      fetch_input()
      execute()
      output_message = @message.join("\n")
      # output(0, output_message)
    rescue => e
      puts({ :status => 'failure', :error => e.message }.to_json)
      exit 1      
    end
  end

  def fetch_input
    params        = JSON.parse(STDIN.read)
    #
    # Use loadpath specfied in the params
    #
    $LOAD_PATH << "#{params['_installdir']}/easy_type/lib"
    $LOAD_PATH << "#{params['_installdir']}/ora_config/lib"
    require 'easy_type'

    #
    # Fetch paramters
    #
    @verbose     = params.fetch('verbose') { 'info' }
    @sid         = params.fetch('sid')
    @format      = params.fetch('format') { 'pretty' }
    @oracle_home = params.fetch('oracle_home')
    @oracle_base = params.fetch('oracle_base')
    @os_user     = params.fetch('os_user')
    @os_group    = params.fetch('os_group')
    @directory   = params.fetch('directory')
    @message     = []
    Puppet::Util::Log.level = @verbose
  end

  def execute
    cd = create_directories
    puts({ :step => 'create directories', :status => 'success', :output => cd }.to_json)
    sm = startup_nomount
    puts({ :step => 'startup nomount', :status => 'success', :output => sm }.to_json)
    rd = restore_database
    puts({ :step => 'restore database', :status => 'success', :output => rd }.to_json)
  end

  def environment
    env = {}
    env[:ORACLE_HOME] = @oracle_home
    env[:ORACLE_SID] = @sid
    env[:PATH] = "/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:#{@oracle_home}/bin"
    env
  end

  def restore_database
    file = File.open("#{@directory}/compatible.txt")
    compatible = file.read
    file = File.open("#{@directory}/source_db.txt")
    source_db = file.read
    restore_commands = [
      'run',
      '{',
      'allocate auxiliary channel c1 device type disk;',
      'allocate auxiliary channel c2 device type disk;',
      'allocate auxiliary channel c3 device type disk;',
      'allocate auxiliary channel c4 device type disk;',
      'allocate auxiliary channel c5 device type disk;',
      "duplicate database to #{@sid}",
      "spfile",
      "parameter_value_convert ('#{source_db}', '#{@sid}')",
      "set compatible='#{compatible}'",
      "nofilenamecheck",
      "backup location '#{@directory}';",
      "}"
    ]
    restore = restore_commands.join("\n")
    command = "echo \"#{restore}\" | #{@oracle_home}/bin/rman auxiliary /"
    Puppet::Util::Execution.execute(command, :failonfail => true, :custom_environment => environment, :uid => @os_user)
  end

  def startup_nomount
    File.open("#{@directory}/init_dup.ora", "w") { |f| f.write "db_name=#{@sid}" }
    FileUtils.chown @os_user, -1, "#{@directory}/init_dup.ora"
    sql_command = "startup nomount pfile=#{@directory}/init_dup.ora"
    command = "echo #{sql_command} | sqlplus / as sysdba"
    Puppet::Util::Execution.execute(command, :failonfail => true, :custom_environment => environment, :uid => @os_user)
  end

  def create_directories
    directories.each {|d| make_oracle_directory d }
  end

  def make_oracle_directory(path)
    Puppet.debug "Creating directory #{path}"
    FileUtils.mkdir_p path
    owned_by_oracle(path)
  end

  def owned_by_oracle(*path)
    Puppet.debug "directories.rb: Setting ownership for #{path}"
    if Puppet::Util::Platform.windows?
      command = "#{@oracle_home}\\bin\\oradim.exe -ACL -setperm diag -USER ORA_#{oracle_home_name}_SVCACCTS -OBJTYPE dir -OBJPATH #{path[0]} -RECURSE false"
      Puppet::Util::Execution.execute(command)
    else
      FileUtils.chmod 0o775, path
      FileUtils.chown @os_user, @os_group, path
    end
  end

  def sep
    if Puppet::Util::Platform.windows?
      '\\'
    else
      '/'
    end
  end

  def confdir
    if Puppet::Util::Platform.windows?
      'database'
    else
      'dbs'
    end
  end

  def directories
    [
    @oracle_base,
    @oracle_home,
    "#{@oracle_home}#{sep}#{confdir}",
    "#{@oracle_base}#{sep}admin",
    "#{@oracle_base}#{sep}cfgtoollogs",
    "#{@oracle_base}#{sep}cfgtoollogs#{sep}dbca",
    "#{@oracle_base}#{sep}cfgtoollogs#{sep}dbca#{sep}#{@sid}",
    ] + named_oracle_directories
  end

  def named_oracle_directories
    [
      "#{@oracle_base}#{sep}admin#{sep}#{@sid}",
      "#{@oracle_base}#{sep}admin#{sep}#{@sid}#{sep}adump",
      "#{@oracle_base}#{sep}admin#{sep}#{@sid}#{sep}ddump",
      "#{@oracle_base}#{sep}admin#{sep}#{@sid}#{sep}dpdump",
      "#{@oracle_base}#{sep}admin#{sep}#{@sid}#{sep}hdump",
      "#{@oracle_base}#{sep}admin#{sep}#{@sid}#{sep}pfile",
      "#{@oracle_base}#{sep}admin#{sep}#{@sid}#{sep}scripts",
      "#{@oracle_base}#{sep}admin#{sep}#{@sid}#{sep}scripts#{sep}log",
    ]
  end

  def oracle_home_name
    reg = Win32::Registry::HKEY_LOCAL_MACHINE.open('SOFTWARE\\Oracle')
    reg.keys.each do |regkey|
      if regkey =~ /^KEY_/
        home_reg = Win32::Registry::HKEY_LOCAL_MACHINE.open("SOFTWARE\\Oracle\\#{regkey}")
        if home_reg['ORACLE_HOME'] == oracle_home
          return home_reg['ORACLE_HOME_NAME']
        end
      end
    end
  end
end

InsertDataBackup.run if __FILE__ == $0

==> enterprisemodules-ora_config-3.18.4.tar.gz: enterprisemodules-ora_config-3.18.4/tasks/insert_data_bu.rb <==
#!/opt/puppetlabs/puppet/bin/ruby
# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'json'
require 'puppet'

begin
  Puppet.initialize_settings
rescue Puppet::DevError
  # When run in Rspec, this is called multiple times
end
# rubocop:disable Style/Documentation
class InsertDataBackup
  # rubocop:enable Style/Documentation
  def self.run
    instance = new
    instance.run
  end

  def run
    @starttime = Time.now.iso8601
    begin
      fetch_input
      execute
      # rubocop:disable Lint/UselessAssignment
      output_message = @message.join("\n")
      # rubocop:enable Lint/UselessAssignment
      # output(0, output_message)
    rescue StandardError => e
      puts({ :status => 'failure', :error => e.message }.to_json)
      exit 1
    end
  end

  def fetch_input
    params        = JSON.parse($stdin.read)
    #
    # Use loadpath specfied in the params
    #
    $LOAD_PATH << "#{params['_installdir']}/easy_type/lib"
    $LOAD_PATH << "#{params['_installdir']}/ora_config/lib"
    require 'easy_type'

    #
    # Fetch paramters
    #
    @verbose     = params.fetch('verbose', 'info')
    @sid         = params.fetch('sid')
    @format      = params.fetch('format', 'pretty')
    @oracle_home = params.fetch('oracle_home')
    @oracle_base = params.fetch('oracle_base')
    @os_user     = params.fetch('os_user')
    @os_group    = params.fetch('os_group')
    @directory   = params.fetch('directory')
    @message     = []
    Puppet::Util::Log.level = @verbose
  end

  def execute
    cd = create_directories
    puts({ :step => 'create directories', :status => 'success', :output => cd }.to_json)
    sm = startup_nomount
    puts({ :step => 'startup nomount', :status => 'success', :output => sm }.to_json)
    rd = restore_database
    puts({ :step => 'restore database', :status => 'success', :output => rd }.to_json)
  end

  def environment
    env = {}
    env[:ORACLE_HOME] = @oracle_home
    env[:ORACLE_SID] = @sid
    env[:PATH] = "/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:#{@oracle_home}/bin"
    env
  end

  def restore_database
    file = File.open("#{@directory}/compatible.txt")
    compatible = file.read
    file = File.open("#{@directory}/source_db.txt")
    source_db = file.read
    restore_commands = [
      'run',
      '{',
      'allocate auxiliary channel c1 device type disk;',
      'allocate auxiliary channel c2 device type disk;',
      'allocate auxiliary channel c3 device type disk;',
      'allocate auxiliary channel c4 device type disk;',
      'allocate auxiliary channel c5 device type disk;',
      "duplicate database to #{@sid}",
      'spfile',
      "parameter_value_convert ('#{source_db}', '#{@sid}')",
      "set compatible='#{compatible}'",
      'nofilenamecheck',
      "backup location '#{@directory}';",
      '}'
    ]
    restore = restore_commands.join("\n")
    command = "echo \"#{restore}\" | #{@oracle_home}/bin/rman auxiliary /"
    Puppet::Util::Execution.execute(command, :failonfail => true, :custom_environment => environment, :uid => @os_user)
  end

  def startup_nomount
    File.open("#{@directory}/init_dup.ora", 'w') { |f| f.write "db_name=#{@sid}" }
    FileUtils.chown @os_user, -1, "#{@directory}/init_dup.ora"
    sql_command = "startup nomount pfile=#{@directory}/init_dup.ora"
    command = "echo #{sql_command} | sqlplus / as sysdba"
    Puppet::Util::Execution.execute(command, :failonfail => true, :custom_environment => environment, :uid => @os_user)
  end

  def create_directories
    directories.each { |d| make_oracle_directory d }
  end

  def make_oracle_directory(path)
    Puppet.debug "Creating directory #{path}"
    FileUtils.mkdir_p path
    owned_by_oracle(path)
  end

  def owned_by_oracle(*path)
    Puppet.debug "directories.rb: Setting ownership for #{path}"
    if Puppet::Util::Platform.windows?
      command = "#{@oracle_home}\\bin\\oradim.exe -ACL -setperm diag -USER ORA_#{oracle_home_name}_SVCACCTS -OBJTYPE dir -OBJPATH #{path[0]} -RECURSE false"
      Puppet::Util::Execution.execute(command)
    else
      FileUtils.chmod 0o775, path
      FileUtils.chown @os_user, @os_group, path
    end
  end

  def sep
    if Puppet::Util::Platform.windows?
      '\\'
    else
      '/'
    end
  end

  def confdir
    if Puppet::Util::Platform.windows?
      'database'
    else
      'dbs'
    end
  end

  def directories
    [
      @oracle_base,
      @oracle_home,
      "#{@oracle_home}#{sep}#{confdir}",
      "#{@oracle_base}#{sep}admin",
      "#{@oracle_base}#{sep}cfgtoollogs",
      "#{@oracle_base}#{sep}cfgtoollogs#{sep}dbca",
      "#{@oracle_base}#{sep}cfgtoollogs#{sep}dbca#{sep}#{@sid}"
    ] + named_oracle_directories
  end

  def named_oracle_directories
    [
      "#{@oracle_base}#{sep}admin#{sep}#{@sid}",
      "#{@oracle_base}#{sep}admin#{sep}#{@sid}#{sep}adump",
      "#{@oracle_base}#{sep}admin#{sep}#{@sid}#{sep}ddump",
      "#{@oracle_base}#{sep}admin#{sep}#{@sid}#{sep}dpdump",
      "#{@oracle_base}#{sep}admin#{sep}#{@sid}#{sep}hdump",
      "#{@oracle_base}#{sep}admin#{sep}#{@sid}#{sep}pfile",
      "#{@oracle_base}#{sep}admin#{sep}#{@sid}#{sep}scripts",
      "#{@oracle_base}#{sep}admin#{sep}#{@sid}#{sep}scripts#{sep}log"
    ]
  end

  def oracle_home_name
    reg = Win32::Registry::HKEY_LOCAL_MACHINE.open('SOFTWARE\\Oracle')
    reg.each_key do |regkey|
      next unless regkey =~ /^KEY_/

      home_reg = Win32::Registry::HKEY_LOCAL_MACHINE.open("SOFTWARE\\Oracle\\#{regkey}")
      return home_reg['ORACLE_HOME_NAME'] if home_reg['ORACLE_HOME'] == oracle_home
    end
  end
end

InsertDataBackup.run if __FILE__ == $PROGRAM_NAME
